/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mnenhy.
 *
 * The Initial Developer of the Original Code is Karsten Düsterloh <mnenhy@tprac.de>.
 *
 * Portions created by the Initial Developer are Copyright (C) 2002-2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Karsten Düsterloh <mnenhy@tprac.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// +-
// |  the contents of this file will be loaded into the Mnenhy.headers.* context!
// |  ie. foo() is available as goMnenhy.headers.foo()
// +-
// |  used Mnenhy preferences:
// |    extensions.mnenhy.headers.viewmode    int     view mode (see contants)
// |    extensions.mnenhy.headers.inline      bool    show message headers inline?
// |    extensions.mnenhy.headers.noxface     bool    don't show X-Face?
// +-

// |
// |  constants
// |
const gkiViewDirty         =  -1; // indicates "view changed!"; GetHeaderView() *NEVER* returns this
const gkiViewBriefHeaders  =   0;
const gkiViewNormalHeaders =   1;
const gkiViewAllHeaders    =   2;
const gkiViewBriefMnenhy   =  90; // instead of gkiViewBriefHeaders; for Enigmail compatibility
const gkiViewNormalMnenhy  =  91; // instead of gkiViewNormalHeaders; for Enigmail compatibility
const gkiViewNoHeaders     =  99;
const gkiViewCustomBase    = 100;

const gksViewElement = "msgHeaderView";
const gksViewClosed  = "collapsed";


// |
// |  header view mode
// |

// if "mnenhy.rdf".general.view.viewmode exists, getHeaderView returns its value;
// otherwise the mail.show_headers value is returned.
function GetHeaderView()
{
  var iViewMode = goMnenhy.GetPref("extensions.mnenhy.headers.viewmode", goMnenhy.kiPrefInt);
  if (iViewMode === null)
    iViewMode = goMnenhy.GetPref("mail.show_headers", goMnenhy.kiPrefInt);
  if (iViewMode == gkiViewAllHeaders)
    ProcessInline(GetHeaderViewInline());
  return Number(iViewMode);
}
// set standard mozilla pref; all mnenhy modes set gkiViewAllHeaders in mozilla
function SetHeaderView(aiMode)
{
  const kiMode = (aiMode < this.gkiViewAllHeaders) ? aiMode : this.gkiViewAllHeaders;
  goMnenhy.SetPref("mail.show_headers", goMnenhy.kiPrefInt, kiMode);
  // set mnenhy pref
  goMnenhy.SetPref("extensions.mnenhy.headers.viewmode", goMnenhy.kiPrefInt, aiMode);
}


// inline flag storage handling
function GetHeaderViewInline()
{
  return goMnenhy.GetPref("extensions.mnenhy.headers.inline", goMnenhy.kiPrefBool) == "true";
}
function SetHeaderViewInline(abInline)
{
  goMnenhy.SetPref("extensions.mnenhy.headers.inline", goMnenhy.kiPrefBool, abInline);
}


// "known" is not custom
function CustomHeaderViewCount()
{
  var iCount = goMnenhy.m_oRDF.GetContainerSize("header") - 1;
  if (iCount < 0)
    iCount = 0;
  return iCount;
}


// process inline state wrt the message sink
function ProcessInline(abInline)
{
  if (("goWin" in this) && goWin)
  {
    const koHeader = goWin.document.getElementById(gksViewElement);
    if (abInline)
    { // disable the message sink, thus forcing inline display
      goWin.msgWindow.msgHeaderSink = null;
      // hide the header pane
      koHeader.setAttribute(gksViewClosed, "true");
    }
    else
    { // restore the message sink, thus disabling inline display
      goWin.msgWindow.msgHeaderSink = goWin.messageHeaderSink;
      // show header pane
      koHeader.removeAttribute(gksViewClosed);
    }
  }
}


// get the custom view name at a given index
function CustomViewName(aiIndex)
{
  return goMnenhy.m_oRDF.GetItemValue("name", ["header:custom" + aiIndex]);
}


// helper functions for CopyCSSRules

// - find a CSS in a document by searching for a filename pattern
function FindCSSStyleSheet(aoDocument, aoCSSExpr)
{
  var oStyleSheet = null;
  if ("styleSheets" in aoDocument)
  {
    const koStyleSheets = aoDocument.styleSheets;
    for (var i = 0; !oStyleSheet && (i < koStyleSheets.length); ++i)
    { // for some strange reason I don't understand,
      // the first use of koStyleSheets[i] does fail... :-/
      const koStyleSheet = koStyleSheets.item(i);
      if (koStyleSheet.href.search(aoCSSExpr) != -1)
        oStyleSheet = koStyleSheet;
    }
  }
  return oStyleSheet;
};
// - find a nsIDOMCSSRule in a stylesheet by searching for a selector pattern
function FindCSSRule(aoStyleSheet, aoRuleExpr)
{
  var oRule = null;
  if ("cssRules" in aoStyleSheet)
  {
    const koCSSRules = aoStyleSheet.cssRules;
    for (var j = 0; !oRule && (j < koCSSRules.length); ++j)
    { // for some strange reason I don't understand,
      // the first use of koCSSRules[j] does fail... :-/
      const koRule = koCSSRules.item(j);
      if (    (koRule.type == koRule.STYLE_RULE)
           && (koRule.selectorText.search(aoRuleExpr) != -1))
        oRule = koRule;
    }
  }
  return oRule;
};

// copy a CSS rule from a specific file of a specific document into a specific file of
// another document, optionally renaming it
//    nsIDOMDocument          aoSourceDocument    the source document
//    string|RegExp           aoSourceCSSExpr     pattern for source CSS filename matching
//    array of string|RegExp  aoSourceRuleExpr    patterns for rule selectorText matching
//    nsIDOMDocument          aoTargetDocument    the target document
//    string|RegExp           aoTargetCSSExpr     pattern for target CSS filename matching
//    array of string         asTargetRuleName    new rule selectorTexts; use old ones if empty
function CopyCSSRules(aoSourceDocument, aoSourceCSSExpr, aroSourceRuleExpr,
                      aoTargetDocument, aoTargetCSSExpr, arsTargetRuleName)
{ // get the stylesheets
  const koSourceCSS = FindCSSStyleSheet(aoSourceDocument, aoSourceCSSExpr);
  const koTargetCSS = FindCSSStyleSheet(aoTargetDocument, aoTargetCSSExpr);
  if (koSourceCSS && koTargetCSS && ("length" in aroSourceRuleExpr) && (aroSourceRuleExpr.length))
  { // have stylesheets and possibly some rules to copy
    for (var k = 0; k < aroSourceRuleExpr.length; ++k)
    {
      const koRule = FindCSSRule(koSourceCSS, aroSourceRuleExpr[k]);
      if (koRule)
      { // found rule: copy it
        var sRule = koRule.cssText;
        if ((k in arsTargetRuleName) && (arsTargetRuleName[k]))
        { // rename the rule
          sRule = arsTargetRuleName[k] + "{" + sRule.split("{")[1];
        }
        koTargetCSS.insertRule(sRule, koTargetCSS.cssRules.length);
      }
    } // loop over rules
  } // have stylesheets
}


// - find the first non-empty value of the asSourceCSSProperty that isn't in arsIgnoreValue
//   by checking aoSourceElement and all its parents
function FindCSSBaseValue(aoSourceElement, asSourceCSSProperty, aosIgnoreValue)
{
  var sValue = "";
  if (aoSourceElement)
  { // search for value
    while (!sValue && aoSourceElement && aoSourceElement.ownerDocument)
    {
      const koWin    = aoSourceElement.ownerDocument.defaultView;
      const koStyles = koWin.getComputedStyle(aoSourceElement, null);
      const ksRule   = koStyles.getPropertyValue(asSourceCSSProperty);
      if (ksRule && !(ksRule in aosIgnoreValue))
        sValue = ksRule;
      else
        aoSourceElement = aoSourceElement.parentNode;
    }
  }
  return sValue;
}



// - find the first non-empty value of the asSourceCSSProperty that isn't in arsIgnoreValue
//   by checking aoSourceElement and all its parents
// - if found, insert a respective rule into aoTargetDocument (see CopyCSSRules for details)
function CopyCSSBaseValue(aoSourceElement,  asSourceCSSProperty, aosIgnoreValue,
                          aoTargetDocument, aoTargetCSSExpr,     asTargetRuleName)
{
  if (aoSourceElement && aoTargetDocument && aoTargetCSSExpr && asTargetRuleName)
  {
    var sValue = FindCSSBaseValue(aoSourceElement, asSourceCSSProperty, aosIgnoreValue);
    if (sValue)
    { // copy it
      const koTargetCSS = FindCSSStyleSheet(aoTargetDocument, aoTargetCSSExpr);
      const ksCSSRule   = asTargetRuleName + "{" + asSourceCSSProperty + ":" + sValue + "}";
      //goMnenhy.Log("SetRule: "+ksCSSRule)
      koTargetCSS.insertRule(ksCSSRule, koTargetCSS.cssRules.length);
    }
  }
}



// |
// |  Init called by the global Mnenhy object when loading this script file
// |
function Init(aoMnenhy)
{ // keep pointer to have the Mnenhy object prototype available
  this.goMnenhy = aoMnenhy;
  // init view mode (if necessary)
  if (goMnenhy.GetPref("extensions.mnenhy.headers.viewmode", goMnenhy.kiPrefInt, true) === null)
    goMnenhy.SetPref("extensions.mnenhy.headers.viewmode", goMnenhy.kiPrefInt, gkiViewCustomBase);
  // load x-face support
  if (!("xface" in goMnenhy.headers) && !goMnenhy.GetPref("extensions.mnenhy.headers.noxface", goMnenhy.kiPrefBool))
  {
    goMnenhy.headers.xface = {};
    if (!goMnenhy.Include("chrome://mnenhy-headers/content/mnenhy-headers-xface.js", goMnenhy.headers.xface))
      delete goMnenhy.headers.xface;
  }
}
